/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.util;

import static org.fortiss.tooling.kernel.extension.data.IConstraintViolation.SEVERITY_DIRECT_COMPARATOR;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.kernel.extension.data.IConstraintViolation;
import org.fortiss.tooling.kernel.extension.data.IConstraintViolation.ESeverity;
import org.fortiss.tooling.kernel.model.constraints.ConstrainedWithChecksum;
import org.fortiss.tooling.kernel.model.constraints.ConstraintsFactory;
import org.fortiss.tooling.kernel.model.constraints.ErrorVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.FailVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.IConstrained;
import org.fortiss.tooling.kernel.model.constraints.IConstraint;
import org.fortiss.tooling.kernel.model.constraints.IConstraintVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.OutdatedVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.SuccessVerificationStatus;
import org.fortiss.tooling.kernel.utils.EcoreUtils;

/**
 * Utility functions for constraints.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 985A54753B82CFBFA82AF1FA54A59124
 */
public class ConstraintsUIUtils {
	/**
	 * @param constraint
	 * @return An "outdated" verification status for <code>constraint</code>.
	 */
	public static OutdatedVerificationStatus
			createOutdatedVerificationStatus(IConstraint constraint) {
		OutdatedVerificationStatus status =
				ConstraintsFactory.eINSTANCE.createOutdatedVerificationStatus();
		// The order if these two lines matter: otherwise we might get to some inconsistent state in
		// case some GUI element (like a table summarizing the status of all constraints) listens to
		// changes to the constraint.
		status.setConstraint(constraint);
		constraint.setVerificationStatus(status);
		return status;
	}

	/** Gathers some statuses by verification status. */
	public static class ClassifiedStatuses {

		/** Failed constraints. */
		public List<FailVerificationStatus> failedStatuses;

		/** Error statuses (constraint could not be checked). */
		public List<ErrorVerificationStatus> errorStatuses;

		/** Outdated statuses. */
		public List<OutdatedVerificationStatus> outdatedStatuses;

		/** Successful constraints. */
		public List<SuccessVerificationStatus> successStatuses;

		/** Constructor. */
		public ClassifiedStatuses() {
			failedStatuses = new ArrayList<FailVerificationStatus>();
			errorStatuses = new ArrayList<ErrorVerificationStatus>();
			outdatedStatuses = new ArrayList<OutdatedVerificationStatus>();
			successStatuses = new ArrayList<SuccessVerificationStatus>();
		}
	}

	/**
	 * @param constrained
	 * @return {@link ClassifiedStatuses} corresponding to <code>constrained</code>.
	 */
	public static ClassifiedStatuses getClassifiedStatuses(IConstrained constrained) {
		ClassifiedStatuses res = new ClassifiedStatuses();
		for(IConstraint constraint : constrained.getConstraints()) {
			IConstraintVerificationStatus status = constraint.getVerificationStatus();
			if(status instanceof FailVerificationStatus) {
				res.failedStatuses.add((FailVerificationStatus)status);
			}
			if(status instanceof ErrorVerificationStatus) {
				res.errorStatuses.add((ErrorVerificationStatus)status);
			}
			if(status instanceof OutdatedVerificationStatus) {
				res.outdatedStatuses.add((OutdatedVerificationStatus)status);
			}
			if(status instanceof SuccessVerificationStatus) {
				res.successStatuses.add((SuccessVerificationStatus)status);
			}
		}
		return res;
	}

	/**
	 * @param c
	 * @return {@link ESeverity} corresponding to the status of the constraint <code>c</code>
	 */
	public static ESeverity getConstraintSeverity(IConstraint c) {
		if(c.getVerificationStatus() instanceof FailVerificationStatus) {
			return ESeverity.ERROR;
		} else if(c.getVerificationStatus() instanceof ErrorVerificationStatus) {
			return ESeverity.ERROR;
		} else if(c.getVerificationStatus() instanceof OutdatedVerificationStatus) {
			return ESeverity.WARNING;
		} else {
			return ESeverity.lowest();
		}
	}

	/**
	 * @param c
	 * @return Severity corresponding to the status of the constraints of <code>c</code>.
	 */
	public static ESeverity getSeverity(IConstrained c) {
		try {
			return c.getConstraints().stream().map(cstr -> getConstraintSeverity(cstr))
					.sorted(IConstraintViolation.SEVERITY_DIRECT_COMPARATOR).findFirst().get();
		} catch(Exception e) {
			return null;
		}
	}

	/**
	 * @param sev
	 * @param modelElement
	 * @return the maximum severity between <code>sev</code> and the severity resulting of possibly
	 *         unsatisfied constraints on <code>modelElement</code>.
	 * 
	 *         Note: if, later on, {@link IConstraintViolation}
	 */
	public static ESeverity augmentSeverityWithConstraintSeverity(ESeverity sev,
			EObject modelElement) {
		if(modelElement instanceof IConstrained) {
			ESeverity sev2 = ConstraintsUIUtils.getSeverity((IConstrained)modelElement);
			if(sev == null || (sev2 != null && SEVERITY_DIRECT_COMPARATOR.compare(sev2, sev) < 0)) {
				sev = sev2;
			}
		}
		return sev == null ? ESeverity.lowest() : sev;
	}

	/**
	 * @param c
	 *            Sends (kernel-specific) refresh notifications to the constrained elements to
	 *            trigger a refresh of the marker decorations.
	 */
	public static void triggerMarkersRefresh(IConstraint c) {
		for(ConstrainedWithChecksum cwc : c.getConstrainedsWithChecksum()) {
			EcoreUtils.postRefreshNotification(cwc.getConstrained());
		}
	}

	/**
	 * @param c
	 * @return Standard text describing the status of <code>c</code>.
	 */
	public static String getText(IConstraint c) {
		IConstraintVerificationStatus status = c.getVerificationStatus();
		String mainMsg = "ERROR";
		if(status instanceof SuccessVerificationStatus) {
			mainMsg = "SUCCESS";
		} else if(status instanceof FailVerificationStatus) {
			mainMsg = "FAIL";
		} else if(status instanceof OutdatedVerificationStatus) {
			mainMsg = "OUTDATED";
		}
		return mainMsg;
	}

	/**
	 * @param c
	 * @return Standard colour corresponding to the status of <code>c</code>.
	 */
	public static Color getColor(IConstraint c) {
		Display display = Display.getCurrent();
		IConstraintVerificationStatus status = c.getVerificationStatus();
		if(status instanceof FailVerificationStatus || status instanceof ErrorVerificationStatus) {
			return display.getSystemColor(SWT.COLOR_RED);
		} else if(status instanceof SuccessVerificationStatus) {
			return display.getSystemColor(SWT.COLOR_GREEN);
		}
		return display.getSystemColor(SWT.COLOR_GRAY);
	}
}
